import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { BasicDialogE2EComponent } from '../../../components/basic-dialog/basic-dialog.e2e.component';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { VorgangE2E, VorgangStatusE2E, vorgangStatusLabelE2E } from '../../../model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, exist, haveText, notExist } from '../../../support/cypress.util';
import { loginAsLudwig, loginAsSabine } from '../../../support/user-util';
import {
  createLoeschAnforderungAttachedItem,
  initVorgangAttachedItem,
} from '../../../support/vorgang-attached-item-util';
import { buildVorgang, initVorgang, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Vorgang endgültig löschen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();
  const basicDialog: BasicDialogE2EComponent = mainPage.getBasicDialog();

  const vorgangPage: VorgangPage = new VorgangPage();
  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();

  const vorgangId: string = objectIds[0];

  const vorgangZuLoeschen: VorgangE2E = {
    ...buildVorgang(vorgangId, 'DoZuLoeschenEndgueltigLoeschen'),
    status: VorgangStatusE2E.ZU_LOESCHEN,
  };

  after(() => {
    dropCollections();
  });

  describe('Endgültig Löschen (ludwig löscht den Vorgang für den die sabine die Löschanforderung gestellt hat)', () => {
    before(() => {
      initVorgang(vorgangZuLoeschen);
      initVorgangAttachedItem([createLoeschAnforderungAttachedItem(objectIds[1], vorgangId)]);

      loginAsLudwig();

      waitForSpinnerToDisappear();
      exist(vorgangList.getRoot());
    });

    it('Open Vorgang zum Löschen', () => {
      vorgangList.getListItem(vorgangZuLoeschen.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should have status Zu Löschen', () => {
      haveText(
        vorgangPage.getVorgangDetailHeader().getStatus(),
        vorgangStatusLabelE2E[vorgangZuLoeschen.status],
      );
    });

    it('should show Endgültig Löschen Bestätigungsdialog', () => {
      vorgangFormularButtons.getEndgueltigLoeschenButton().click();
      waitForSpinnerToDisappear();

      exist(basicDialog.getHeader());
    });

    it('should show header text in Bestätigungsdialog', () => {
      contains(basicDialog.getHeader(), 'Möchten Sie den Vorgang endgültig löschen?');
    });

    it('should show warning message in Bestätigungsdialog', () => {
      contains(basicDialog.getMessage(), 'Diese Aktion kann nicht widerrufen werden.');
    });

    it('should have confirmation button', () => {
      exist(basicDialog.getConfirmationButton());
    });

    it('should have cancel button', () => {
      exist(basicDialog.getCancelButton());
    });

    it('should cancel das Löschen', () => {
      basicDialog.getCancelButton().click();
      vorgangPage.getSubnavigation().getBackButton().click();
      waitForSpinnerToDisappear();
      vorgangList.getListItem(vorgangZuLoeschen.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
      haveText(
        vorgangPage.getVorgangDetailHeader().getStatus(),
        vorgangStatusLabelE2E[vorgangZuLoeschen.status],
      );
    });

    it('should be redirected to vorgang list after endgültig löschen', () => {
      vorgangFormularButtons.getEndgueltigLoeschenButton().click();

      waitForSpinnerToDisappear();
      basicDialog.getConfirmationButton().click();
      waitForSpinnerToDisappear();

      exist(vorgangList.getRoot());
    });

    it('vorgang should not exist', () => {
      notExist(vorgangList.getListItem(vorgangZuLoeschen.name).getRoot());
    });
  });

  describe('Endgültig Löschen (sabine löscht den Vorgang für den sabine die Löschanforderung gestellt hat)', () => {
    before(() => {
      initVorgang(vorgangZuLoeschen);
      initVorgangAttachedItem([createLoeschAnforderungAttachedItem(objectIds[1], vorgangId)]);

      loginAsSabine();

      waitForSpinnerToDisappear();
      exist(vorgangList.getRoot());
    });

    it('Open Vorgang zum Löschen', () => {
      vorgangList.getListItem(vorgangZuLoeschen.name).getRoot().click();
      waitForSpinnerToDisappear();

      exist(vorgangPage.getVorgangDetailHeader().getRoot());
    });

    it('should have status Zu Löschen', () => {
      haveText(
        vorgangPage.getVorgangDetailHeader().getStatus(),
        vorgangStatusLabelE2E[vorgangZuLoeschen.status],
      );
    });

    it('should not show Endgültig Löschen button', () => {
      notExist(vorgangFormularButtons.getEndgueltigLoeschenButton());
    });
  });
});
