/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { SnackBarE2EComponent } from 'apps/alfa-e2e/src/components/ui/snackbar.e2e.component';
import { WiedervorlageSubnavigationE2EComponent } from 'apps/alfa-e2e/src/components/wiedervorlage/wiedervorlage-subnavigation';
import { VorgangE2E, VorgangStatusE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { MainPage, waitForSpinnerToDisappear } from 'apps/alfa-e2e/src/page-objects/main.po';
import { WiedervorlagePage } from 'apps/alfa-e2e/src/page-objects/wiedervorlage.po';
import { visitUrl } from 'apps/alfa-e2e/src/support/cypress-helper';
import { contains, exist, notExist } from 'apps/alfa-e2e/src/support/cypress.util';
import { MessagesE2E } from 'apps/alfa-e2e/src/support/messages';
import { UserRoleE2E, loginAsPeter } from 'apps/alfa-e2e/src/support/user-util';
import { buildVorgang, initVorgaenge } from 'apps/alfa-e2e/src/support/vorgang-util';
import { createWiedervorlageNeuUriByVorgangId } from 'apps/alfa-e2e/src/support/wiedervorlage-util';
import { dropCollections } from '../../../support/cypress-helper';

describe('Wiedervorlage should be authorized by role', () => {
  const mainPage: MainPage = new MainPage();
  const snackbar: SnackBarE2EComponent = mainPage.getSnackBar();

  const wiedervorlagePage: WiedervorlagePage = new WiedervorlagePage();
  const wiedervorlageSubnavigation: WiedervorlageSubnavigationE2EComponent =
    wiedervorlagePage.getSubnavigation();

  describe(`for user Peter with role ${UserRoleE2E.VERWALTUNG_POSTSTELLE}`, () => {
    const vorgangInStatusNeuId: string = '601bc78eabc134051570aab8';
    const vorgangInStatusNeuUrl: string =
      createWiedervorlageNeuUriByVorgangId(vorgangInStatusNeuId);
    const vorgangInStatusNeu: VorgangE2E = {
      ...buildVorgang(vorgangInStatusNeuId, 'VorgangInStatusNeu'),
      status: VorgangStatusE2E.NEU,
    };

    const vorgangInStatusInBearbeitungId = '60250b9d383c182943f6ba79';
    const vorgangInStatusInBearbeitungUrl: string = createWiedervorlageNeuUriByVorgangId(
      vorgangInStatusInBearbeitungId,
    );
    const vorgangInStatusInBearbeitung: VorgangE2E = {
      ...buildVorgang(vorgangInStatusInBearbeitungId, 'vorgangInStatusInBearbeitung'),
      status: VorgangStatusE2E.IN_BEARBEITUNG,
    };

    const vorgangInStatusAbgeschlossenId: string = '602566a807bb665df9a86e87';
    const vorgangInStatusAbgeschlossenUrl: string = createWiedervorlageNeuUriByVorgangId(
      vorgangInStatusAbgeschlossenId,
    );
    const vorgangInStatusAbgeschlossen: VorgangE2E = {
      ...buildVorgang(vorgangInStatusAbgeschlossenId, 'vorgangInStatusAbgeschlossen'),
      status: VorgangStatusE2E.ABGESCHLOSSEN,
    };

    const vorgangInStatusBeschiedenId: string = '602566a207bb665df9a86e86';
    const vorgangInStatusBeschiedenUrl: string = createWiedervorlageNeuUriByVorgangId(
      vorgangInStatusBeschiedenId,
    );
    const vorgangInStatusBeschieden: VorgangE2E = {
      ...buildVorgang(vorgangInStatusBeschiedenId, 'vorgangInStatusBeschieden'),
      status: VorgangStatusE2E.BESCHIEDEN,
    };

    beforeEach(() => {
      initVorgaenge([
        vorgangInStatusNeu,
        vorgangInStatusInBearbeitung,
        vorgangInStatusAbgeschlossen,
        vorgangInStatusBeschieden,
      ]);

      loginAsPeter();
    });

    after(() => {
      dropCollections();
    });

    it(`should NOT show vorgang in status ${VorgangStatusE2E.NEU}`, () => {
      visitUrl(vorgangInStatusNeuUrl);
      waitForSpinnerToDisappear();

      exist(mainPage.getHeader().getLogo());
      notExist(wiedervorlagePage.getRoot());
    });

    describe('should hide vorgang in status', () => {
      it(`${VorgangStatusE2E.IN_BEARBEITUNG}`, () => {
        visitUrl(vorgangInStatusInBearbeitungUrl);
        waitForSpinnerToDisappear();

        exist(mainPage.getHeader().getLogo());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
        notExist(wiedervorlagePage.getRoot());
      });

      it(`${VorgangStatusE2E.ABGESCHLOSSEN}`, () => {
        visitUrl(vorgangInStatusAbgeschlossenUrl);
        waitForSpinnerToDisappear();

        exist(mainPage.getHeader().getLogo());
        notExist(wiedervorlagePage.getRoot());
        notExist(wiedervorlageSubnavigation.getRoot());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
      });

      it(`${VorgangStatusE2E.BESCHIEDEN}`, () => {
        visitUrl(vorgangInStatusBeschiedenUrl);
        waitForSpinnerToDisappear();

        exist(mainPage.getHeader().getLogo());
        notExist(wiedervorlagePage.getRoot());
        notExist(wiedervorlageSubnavigation.getRoot());
        contains(snackbar.getMessage(), MessagesE2E.HTTP_STATUS_FORBIDDEN);
      });
    });
  });
});
