import {
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Observable, of } from 'rxjs';
import { singleCold } from '../../../tech-shared/test/marbles';
import { createSettingsListResource } from '../../test/admin-settings';
import { createPostfachResource, createSettingItemResource } from '../../test/postfach/postfach';
import { SettingListResourceService } from './admin-settings-resource.service';
import { SettingListResource } from './admin-settings.model';
import { SettingsService } from './admin-settings.service';
import { PostfachResource } from './postfach/postfach.model';

describe('SettingsService', () => {
  let service: SettingsService;
  let settingListResourceService: Mock<SettingListResourceService>;

  beforeEach(() => {
    settingListResourceService = mock(SettingListResourceService);

    service = new SettingsService(useFromMock(settingListResourceService));
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  describe('get Postfach', () => {
    const postfachResource = createPostfachResource();
    const postfachStateResource: StateResource<PostfachResource> =
      createStateResource(postfachResource);
    const settingsListResource: StateResource<SettingListResource> = createStateResource(
      createSettingsListResource([postfachResource]),
    );

    beforeEach(() => {
      settingListResourceService.getList = jest.fn().mockReturnValue(of(settingsListResource));
    });

    it('should call resource service', () => {
      service.getPostfach();

      expect(settingListResourceService.getList).toHaveBeenCalled();
    });

    it('should return null for non postfach resource', () => {
      const emptySettingsListResource: StateResource<SettingListResource> = createStateResource(
        createSettingsListResource([createSettingItemResource()]),
      );

      settingListResourceService.getList = jest
        .fn()
        .mockReturnValue(singleCold(emptySettingsListResource));

      const postfach: Observable<StateResource<PostfachResource>> = service.getPostfach();
      expect(postfach).toBeObservable(singleCold(createEmptyStateResource()));
    });

    it('should return item resource as postfach resource', () => {
      settingListResourceService.getList = jest
        .fn()
        .mockReturnValue(singleCold(settingsListResource));

      const postfach: Observable<StateResource<PostfachResource>> = service.getPostfach();

      expect(postfach).toBeObservable(singleCold(postfachStateResource));
    });
  });
});
