import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { MockComponent } from 'ng-mocks';
import { singleCold, singleHot } from '../../../../../tech-shared/test/marbles';
import { Mock, mock } from '../../../../../test-utils/src/lib/mocking';
import { createPostfachResource } from '../../../../test/postfach/postfach';
import { PostfachResource } from '../postfach.model';
import { PostfachService } from '../postfach.service';
import { PostfachContainerComponent } from './postfach-container.component';
import { PostfachFormComponent } from './postfach-form/postfach-form.component';

describe('PostfachContainerComponent', () => {
  let component: PostfachContainerComponent;
  let fixture: ComponentFixture<PostfachContainerComponent>;

  const postfachService: Mock<PostfachService> = mock(PostfachService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [PostfachContainerComponent, MockComponent(PostfachFormComponent)],
      imports: [ReactiveFormsModule],
      providers: [
        {
          provide: PostfachService,
          useValue: postfachService,
        },
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PostfachContainerComponent);
    component = fixture.componentInstance;

    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call postfach service', () => {
      component.ngOnInit();

      expect(postfachService.get).toHaveBeenCalled();
    });

    it('should set postfach', () => {
      const postfachStateResource: StateResource<PostfachResource> =
        createStateResource(createPostfachResource());
      postfachService.get.mockReturnValue(singleHot(postfachStateResource));

      component.ngOnInit();

      expect(component.postfachStateResource$).toBeObservable(singleCold(postfachStateResource));
    });
  });
});
