import { HttpError, StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { Type } from '@angular/core';
import { Observable } from 'rxjs';
import { singleCold, singleHot } from '../../../../tech-shared/test/marbles';
import { createPostfachResource, createPostfachSettingItem } from '../../../test/postfach/postfach';
import { PostfachResourceService } from './postfach-resource.service';
import { PostfachResource, PostfachSettingsItem } from './postfach.model';
import { PostfachService } from './postfach.service';

describe('PostfachService', () => {
  let service: PostfachService;
  let resourceService: Mock<PostfachResourceService>;

  beforeEach(() => {
    resourceService = mockResourceService(PostfachResourceService);

    service = new PostfachService(useFromMock(resourceService));
  });

  it('should create', () => {
    expect(service).toBeTruthy();
  });

  describe('get', () => {
    const postfachResource: PostfachResource = createPostfachResource();
    const postfachStateResource: StateResource<PostfachResource> =
      createStateResource(postfachResource);

    it('should call resourceservice get', () => {
      service.get();

      expect(resourceService.get).toHaveBeenCalled();
    });

    it('should reurn value', () => {
      resourceService.get.mockReturnValue(singleCold(postfachStateResource));

      const returnedPostfachResource: Observable<StateResource<PostfachResource | HttpError>> =
        service.get();

      expect(returnedPostfachResource).toBeObservable(singleHot(postfachStateResource));
    });
  });

  describe('save', () => {
    const postfachSettingsItem: PostfachSettingsItem = createPostfachSettingItem();

    it('should call resourceService', () => {
      service.save(postfachSettingsItem.settingBody);

      expect(resourceService.save).toHaveBeenCalledWith(postfachSettingsItem);
    });

    it('should return saved value', () => {
      const postfachResource: PostfachResource = createPostfachResource();
      const postfachStateResource: StateResource<PostfachResource> =
        createStateResource(postfachResource);
      resourceService.save.mockReturnValue(singleCold(postfachStateResource));

      const savedPostfach: Observable<StateResource<PostfachResource | HttpError>> = service.save(
        postfachResource.settingBody,
      );

      expect(savedPostfach).toBeObservable(singleHot(postfachStateResource));
    });
  });
});

export function mockResourceService<T>(service: Type<T>): Mock<T> {
  return <Mock<T>>{ ...mock(service), get: jest.fn(), save: jest.fn() };
}
