import { BescheidService } from '@alfa-client/bescheid-shared';
import { BinaryFileUriContainerComponent } from '@alfa-client/binary-file';
import {
  GetUrlPipe,
  HasLinkPipe,
  StateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import {
  Mock,
  existsAsHtmlElement,
  getMockComponent,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import faker from '@faker-js/faker';
import { ResourceUri, getUrl } from '@ngxp/rest';
import { AttachmentWrapperComponent } from '@ods/system';
import { DocumentLinkRel } from 'libs/bescheid-shared/src/lib/document.linkrel';
import { DocumentResource } from 'libs/bescheid-shared/src/lib/document.model';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createDocumentResource } from '../../../../../../bescheid-shared/src/test/document';
import { DocumentInBescheidContainerComponent } from './document-in-bescheid-container.component';

describe('DocumentInBescheidContainerComponent', () => {
  let component: DocumentInBescheidContainerComponent;
  let fixture: ComponentFixture<DocumentInBescheidContainerComponent>;

  const binaryFileUriContainer: string = getDataTestClassOf('binary-file-uri-container');

  const bescheidService: Mock<BescheidService> = mock(BescheidService);

  const documentUri: ResourceUri = faker.internet.url();

  const document: DocumentResource = createDocumentResource([DocumentLinkRel.FILE]);
  const documentStateResource: StateResource<DocumentResource> = createStateResource(document);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        DocumentInBescheidContainerComponent,
        HasLinkPipe,
        GetUrlPipe,
        MockComponent(BinaryFileUriContainerComponent),
        MockComponent(AttachmentWrapperComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(DocumentInBescheidContainerComponent);
    component = fixture.componentInstance;
    component.documentUri = documentUri;
    bescheidService.loadBescheidDocument.mockReturnValue(of(documentStateResource));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service to load bescheid document', () => {
      component.ngOnInit();

      expect(bescheidService.loadBescheidDocument).toHaveBeenCalledWith(documentUri);
    });
  });

  describe('binary file uri container', () => {
    it('should be visible if link exists on resource', () => {
      component.documentStateResource$ = of(
        createStateResource(createDocumentResource([DocumentLinkRel.FILE])),
      );

      fixture.detectChanges();

      existsAsHtmlElement(fixture, binaryFileUriContainer);
    });

    it('should be hidden if link is not presetn on resource', () => {
      component.documentStateResource$ = of(createStateResource(createDocumentResource()));

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, binaryFileUriContainer);
    });

    it('should be called with binaryFileUri', () => {
      const binaryFileUriContainer: BinaryFileUriContainerComponent =
        getMockComponent<BinaryFileUriContainerComponent>(fixture, BinaryFileUriContainerComponent);

      expect(binaryFileUriContainer.binaryFileUri).toBe(getUrl(document, DocumentLinkRel.FILE));
    });
  });
});
