import { BinaryFileResource, BinaryFileService } from '@alfa-client/binary-file-shared';
import {
  StateResource,
  createEmptyStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import {
  Mock,
  existsAsHtmlElement,
  getMockComponent,
  mock,
  notExistsAsHtmlElement,
} from '@alfa-client/test-utils';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import faker from '@faker-js/faker';
import { ResourceUri } from '@ngxp/rest';
import { createBinaryFileResource } from 'libs/binary-file-shared/test/binary-file';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { BinaryFile2ContainerComponent } from '../binary-file2-container/binary-file2-container.component';
import { BinaryFileUriContainerComponent } from './binary-file-uri-container.component';

import { AttachmentWrapperComponent } from '@ods/system';

describe('BinaryFileUriContainerComponent', () => {
  let component: BinaryFileUriContainerComponent;
  let fixture: ComponentFixture<BinaryFileUriContainerComponent>;

  const binaryFileContainer: string = getDataTestClassOf('binary-file-container');
  const binaryFileUri: ResourceUri = faker.internet.url();

  const binaryFileService: Mock<BinaryFileService> = mock(BinaryFileService);

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BinaryFileUriContainerComponent,
        MockComponent(BinaryFile2ContainerComponent),
        MockComponent(AttachmentWrapperComponent),
      ],
      providers: [
        {
          provide: BinaryFileService,
          useValue: binaryFileService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BinaryFileUriContainerComponent);
    component = fixture.componentInstance;
    component.binaryFileUri = binaryFileUri;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should call service to get file', () => {
      component.ngOnInit();

      expect(binaryFileService.getFile).toHaveBeenCalledWith(binaryFileUri);
    });
  });

  describe('binary file container', () => {
    const binaryFile: BinaryFileResource = createBinaryFileResource();
    const binaryFileStateResource: StateResource<BinaryFileResource> =
      createStateResource(binaryFile);

    beforeEach(() => {
      binaryFileService.getFile.mockReturnValue(of(binaryFileStateResource));
    });

    it('should show if resource exists', () => {
      component.binaryFileStateResource$ = of(createStateResource(createBinaryFileResource()));

      fixture.detectChanges();

      existsAsHtmlElement(fixture, binaryFileContainer);
    });

    it('should hide if resource is null', () => {
      component.binaryFileStateResource$ = of(createEmptyStateResource<BinaryFileResource>());

      fixture.detectChanges();

      notExistsAsHtmlElement(fixture, binaryFileContainer);
    });

    it('should be called with file', () => {
      const binaryFileContainerComponent: BinaryFile2ContainerComponent =
        getMockComponent<BinaryFile2ContainerComponent>(fixture, BinaryFile2ContainerComponent);

      expect(binaryFileContainerComponent.file).toEqual(binaryFileStateResource.resource);
    });

    it('should be called with deleteable', () => {
      const binaryFileContainerComponent: BinaryFile2ContainerComponent =
        getMockComponent<BinaryFile2ContainerComponent>(fixture, BinaryFile2ContainerComponent);

      expect(binaryFileContainerComponent.deletable).toBe(false);
    });
  });
});
