import { CollaborationListResource } from '@alfa-client/collaboration-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { HttpError, StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { UntypedFormBuilder } from '@angular/forms';
import { CollaborationService } from 'libs/collaboration-shared/src/lib/collaboration.service';
import { createCollaborationListResource } from 'libs/collaboration-shared/test/collaboration';
import { createCommandResource } from 'libs/command-shared/test/command';
import { singleColdCompleted } from 'libs/tech-shared/test/marbles';
import { Observable, of } from 'rxjs';
import { CollaborationRequestFormService } from './collaboration.request.formservice';

describe('CollaborationRequestFormService', () => {
  let formService: CollaborationRequestFormService;

  let service: Mock<CollaborationService>;

  const formBuilder: UntypedFormBuilder = new UntypedFormBuilder();

  const collaborationListResource: CollaborationListResource = createCollaborationListResource();

  beforeEach(() => {
    service = mock(CollaborationService);

    formService = new CollaborationRequestFormService(formBuilder, useFromMock(service));
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('do submit', () => {
    const stateCommandResource: StateResource<CommandResource> =
      createStateResource(createCommandResource());

    beforeEach(() => {
      formService.listResource = collaborationListResource;
      service.create.mockReturnValue(of(stateCommandResource));
    });

    it('should call service', () => {
      formService.submit();

      expect(service.create).toHaveBeenCalledWith(
        collaborationListResource,
        formService.form.value,
      );
    });

    it('should return stateCommandResource', () => {
      const response$: Observable<StateResource<CommandResource | HttpError>> =
        formService.submit();

      expect(response$).toBeObservable(singleColdCompleted(stateCommandResource));
    });
  });

  describe('set list resource', () => {
    it('should set given list resource', () => {
      formService.listResource = undefined;

      formService.setListResource(collaborationListResource);

      expect(formService.listResource).toBe(collaborationListResource);
    });
  });
});
