import {
  OrganisationsEinheitListLinkRel,
  OrganisationsEinheitService,
} from '@alfa-client/collaboration-shared';
import { StateResource } from '@alfa-client/tech-shared';
import { DialogRef } from '@angular/cdk/dialog';
import { Component, HostListener, OnInit } from '@angular/core';
import {
  OrganisationsEinheitListResource,
  OrganisationsEinheitResource,
} from 'libs/collaboration-shared/src/lib/organisations-einheit.model';
import { Observable } from 'rxjs';

@Component({
  selector: 'alfa-search-organisations-einheit-container',
  templateUrl: './search-organisations-einheit-container.component.html',
})
export class SearchOrganisationsEinheitContainerComponent implements OnInit {
  public organisationsEinheitStateListResource$: Observable<
    StateResource<OrganisationsEinheitListResource>
  >;

  public readonly organisationsEinheitListLinkRel = OrganisationsEinheitListLinkRel;

  constructor(
    private readonly service: OrganisationsEinheitService,
    private readonly dialogRef: DialogRef,
  ) {}

  ngOnInit(): void {
    this.organisationsEinheitStateListResource$ = this.service.getSearchResultList();
  }

  @HostListener('document:keydown', ['$event']) onKeyDownHandler(e: KeyboardEvent) {
    if (e.key === 'Enter') e.preventDefault();
  }

  public search(searchBy: string): void {
    this.service.search(searchBy);
  }

  public selectSearchResult(organisationsEinheit: OrganisationsEinheitResource): void {
    this.service.selectSearchResult(organisationsEinheit);
    this.service.clearSearchResult();
    this.dialogRef.close();
  }

  public clearSearchResult(): void {
    this.service.clearSearchResult();
  }

  public closeDialog(): void {
    this.clearSearchResult();
    this.dialogRef.close();
  }
}
