import { CommonModule } from '@angular/common';
import { Component, Input } from '@angular/core';

import { SpinnerIconComponent } from '../icons/spinner-icon/spinner-icon.component';

@Component({
  selector: 'ods-button-card',
  standalone: true,
  imports: [CommonModule, SpinnerIconComponent],
  styles: [':host {@apply inline-flex}'],
  template: `<button
    type="button"
    class="flex flex-grow items-center justify-center gap-4 rounded-md bg-background-50 py-3 pl-6 pr-6 text-text hover:bg-background-100 focus:outline-none  focus-visible:outline focus-visible:outline-2 focus-visible:outline-offset-2 focus-visible:outline-ozgblue-800 disabled:cursor-wait disabled:hover:bg-background-50"
    [disabled]="isLoading"
    [attr.aria-disabled]="isLoading"
    [attr.aria-label]="getAriaLabel()"
  >
    <ng-content *ngIf="!isLoading" select="[icon]"></ng-content>
    <ods-spinner-icon *ngIf="isLoading" size="extra-large" class="shrink-0" />
    <div class="flex-grow break-all">
      {{ text }}
      <br *ngIf="subText" aria-hidden="true" />
      {{ subText }}
    </div>
  </button>`,
})
export class ButtonCardComponent {
  @Input({ required: true }) text!: string;
  @Input() subText: string = '';
  @Input() isLoading: boolean = false;

  getAriaLabel() {
    return this.text + (this.subText ? ` ${this.subText}` : '');
  }
}
