/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UntypedFormBuilder } from '@angular/forms';
import { CommandResource } from '@alfa-client/command-shared';
import { PostfachService } from '@alfa-client/postfach-shared';
import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { createCommandResource } from 'libs/command-shared/test/command';
import { createPostfachMailResource } from 'libs/postfach-shared/test/postfach';
import { of } from 'rxjs';
import { PostfachMailFormservice } from './postfach-mail.formservice';

describe('PostfachMailFormservice', () => {
  let formService: PostfachMailFormservice;
  let service: Mock<any>;

  const formBuilder: UntypedFormBuilder = new UntypedFormBuilder();

  beforeEach(() => {
    service = mock(PostfachService);

    formService = new PostfachMailFormservice(formBuilder, useFromMock(service));
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('submit', () => {
    const commandStateResource: StateResource<CommandResource> =
      createStateResource(createCommandResource());

    beforeEach(() => {
      service.doSubmit = jest.fn();
    });

    describe('on patch', () => {
      beforeEach(() => {
        service.sendExistingMail.mockReturnValue(of(commandStateResource));
      });

      it('should call service', () => {
        formService.source = createPostfachMailResource();

        formService.submit();

        expect(service.sendExistingMail).toHaveBeenCalledWith(
          formService.source,
          formService.form.value,
        );
      });
    });

    describe('on new', () => {
      beforeEach(() => {
        service.sendMail.mockReturnValue(of(commandStateResource));
      });

      it('should call service', () => {
        formService.submit();

        expect(service.sendMail).toHaveBeenCalledWith(formService.form.value);
      });
    });
  });
});
