import { Component, Input, OnDestroy } from '@angular/core';
import { BinaryFileResource } from '@alfa-client/binary-file-shared';
import {
  PostfachMailLinkRel,
  PostfachMailListLinkRel,
  PostfachMailListResource,
  PostfachMailResource,
  PostfachService,
} from '@alfa-client/postfach-shared';
import { EMPTY_ARRAY, FormProvider, StateResource, isNotNil } from '@alfa-client/tech-shared';
import { hasLink } from '@ngxp/rest';
import { Observable, of } from 'rxjs';
import { PostfachMailFormservice } from '../postfach-mail.formservice';

@Component({
  selector: 'alfa-postfach-nachricht-attachment-container',
  templateUrl: './postfach-nachricht-attachment-container.component.html',
  styleUrls: ['./postfach-nachricht-attachment-container.component.scss'],
  viewProviders: [FormProvider],
})
export class PostfachNachrichtAttachmentContainerComponent implements OnDestroy {
  @Input() postfachNachricht: PostfachMailResource;

  postfachNachrichtListStateResource$: Observable<StateResource<PostfachMailListResource>>;
  attachments$: Observable<BinaryFileResource[]> = of<BinaryFileResource[]>(EMPTY_ARRAY);

  public readonly postfachMailListLinkRel = PostfachMailListLinkRel;
  public readonly formServiceClass = PostfachMailFormservice;

  constructor(private postfachService: PostfachService) {}

  ngOnInit(): void {
    this.postfachNachrichtListStateResource$ = this.postfachService.getPostfachMailListByVorgang();

    if (isNotNil(this.postfachNachricht)) {
      this.initAttachments();
    }
  }

  initAttachments(): void {
    if (hasLink(this.postfachNachricht, PostfachMailLinkRel.ATTACHMENTS)) {
      this.attachments$ = this.postfachService.getAttachments(this.postfachNachricht);
    }
  }

  ngOnDestroy(): void {
    this.postfachService.clearAttachmentList();
  }
}
