import { ApiError, getMessageForIssue } from '@alfa-client/tech-shared';
import { createApiError } from '../../../test/error';
import { ConvertApiErrorToErrorMessagesPipe } from './convert-api-error-to-error-messages.pipe';

jest.mock('@alfa-client/tech-shared');
const getMessageForIssueMock = getMessageForIssue as jest.Mock;

describe('convertApiErrorToErrorMessagesPipe', () => {
  const pipe = new ConvertApiErrorToErrorMessagesPipe();

  it('create an instance', () => {
    expect(pipe).toBeTruthy();
  });

  describe('transform', () => {
    it.each([null, undefined])('should return empty array for %s value', (value: ApiError) => {
      const errorMessages: string[] = pipe.transform(value);

      expect(errorMessages).toEqual([]);
    });

    it('should get message for issue', () => {
      pipe.transform(createApiError());

      expect(getMessageForIssueMock).toHaveBeenCalled();
    });

    it('should return array of error messages', () => {
      const expectedErrorMessage = 'Fehler';
      getMessageForIssueMock.mockReturnValue(expectedErrorMessage);

      const errorMessages: string[] = pipe.transform(createApiError());

      expect(errorMessages).toEqual([expectedErrorMessage]);
    });
  });
});
