/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { ConvertForDataTestPipe } from '@alfa-client/tech-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule } from '@angular/forms';
import { MatNativeDateModule } from '@angular/material/core';
import { MatDatepickerModule } from '@angular/material/datepicker';
import { MatFormFieldModule } from '@angular/material/form-field';
import { MatIconModule } from '@angular/material/icon';
import { MatInputModule } from '@angular/material/input';
import { BrowserAnimationsModule } from '@angular/platform-browser/animations';
import faker from '@faker-js/faker';
import { MockComponent } from 'ng-mocks';
import { ValidationErrorComponent } from '../../validation-error/validation-error.component';
import { DateEditorComponent } from './date-editor.component';

import * as dateUtil from '../../../../../../tech-shared/src/lib/date.util';

describe('DateEditorComponent', () => {
  let component: DateEditorComponent;
  let fixture: ComponentFixture<DateEditorComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        DateEditorComponent,
        ConvertForDataTestPipe,
        MockComponent(ValidationErrorComponent),
      ],
      imports: [
        MatFormFieldModule,
        MatInputModule,
        MatIconModule,
        MatDatepickerModule,
        MatNativeDateModule,
        ReactiveFormsModule,
        BrowserAnimationsModule,
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(DateEditorComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('data test id', () => {
    it('should exist for input', () => {
      component.label = 'Ein Label';
      fixture.detectChanges();

      const element: HTMLElement = fixture.nativeElement.querySelector(
        '[data-test-id="Ein_Label-date-input"]',
      );

      expect(element).toBeInstanceOf(HTMLElement);
    });
  });

  describe('onBlur', () => {
    it('should not call add2000Years if input value is not Date object', () => {
      const inputValue: string = '12.12.2024';
      component.fieldControl.setValue(inputValue);
      const add2000Years = jest.spyOn(dateUtil, 'add2000Years');

      component.onBlur();

      expect(add2000Years).not.toHaveBeenCalled();
    });

    describe('if input value is Date object', () => {
      const inputValue: Date = faker.date.recent();

      beforeEach(() => {
        component.fieldControl.setValue(inputValue);
      });

      it('should call add2000Years', () => {
        const add2000Years = jest.spyOn(dateUtil, 'add2000Years');

        component.onBlur();

        expect(add2000Years).toHaveBeenCalledWith(inputValue);
      });

      it('should call fieldControl.patchValue', () => {
        const patchValue = jest.spyOn(component.fieldControl, 'patchValue');

        component.onBlur();

        expect(patchValue).toHaveBeenCalled();
      });

      it('should call onChange', () => {
        const onChange = jest.spyOn(component, 'onChange');

        component.onBlur();

        expect(onChange).toHaveBeenCalled();
      });
    });
  });
});
