import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { getDataTestClassOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { SpinnerComponent } from '../../../spinner/spinner.component';
import { OzgcloudButtonContentComponent } from './ozgcloud-button-content.component';

describe('OzgcloudButtonContentComponent', () => {
  let component: OzgcloudButtonContentComponent;
  let fixture: ComponentFixture<OzgcloudButtonContentComponent>;

  const iconSelector: string = getDataTestClassOf('icon');
  const text: string = getDataTestClassOf('button-with-spinner-text');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [MatIcon, MockComponent(SpinnerComponent), OzgcloudButtonContentComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(OzgcloudButtonContentComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('Icon', () => {
    beforeEach(() => {
      component.icon = 'add';
    });

    it('should be visible', () => {
      component.isDisabled = false;
      fixture.detectChanges();

      const icon = fixture.nativeElement.querySelector(iconSelector);

      expect(icon).toBeVisible();
    });

    it('should not be visible', () => {
      component.isDisabled = true;
      fixture.detectChanges();

      const icon = fixture.nativeElement.querySelector(iconSelector);

      expect(icon).not.toBeVisible();
    });
  });

  describe('Text', () => {
    it('should be visible if exist', () => {
      component.text = 'Test text';
      fixture.detectChanges();

      const textElement = fixture.nativeElement.querySelector(text);

      expect(textElement).toBeInTheDocument();
    });

    it('should not be visible if is null', () => {
      component.text = null;
      fixture.detectChanges();

      const textElement = fixture.nativeElement.querySelector(text);

      expect(textElement).not.toBeInTheDocument();
    });
  });
});
