/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { mock, useFromMock } from '@alfa-client/test-utils';
import { UserProfileLinkRel, UserProfileResource } from '@alfa-client/user-profile-shared';
import { ResourceFactory } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import { createUserProfileResource } from 'libs/user-profile-shared/test/user-profile';
import {
  createUserSettings,
  createUserSettingsResource,
} from 'libs/user-settings-shared/test/user-settings';
import { UserSettingsLinkRel } from './user-settings.linkrel';
import { UserSettings, UserSettingsResource } from './user-settings.model';
import { UserSettingsRepository } from './user-settings.repository';

describe('UserSettingsRepository', () => {
  let repository: UserSettingsRepository;
  let resourceFactory = mock(ResourceFactory);
  let resourceWrapper = { get: jest.fn(), patch: jest.fn() };

  beforeEach(() => {
    repository = new UserSettingsRepository(useFromMock(resourceFactory));

    resourceFactory.from.mockReturnValue(resourceWrapper);
  });

  describe('get user settings', () => {
    const userProfileResource: UserProfileResource = createUserProfileResource();
    const responseResource: UserProfileResource = createUserProfileResource();

    beforeEach(() => {
      resourceWrapper.get.mockReturnValue(hot('a', { a: responseResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.getUserSettings(userProfileResource);

      expect(resourceFactory.from).toHaveBeenCalledWith(userProfileResource);
    });

    it('should call resourceWrapper with link', () => {
      repository.getUserSettings(userProfileResource);

      expect(resourceWrapper.get).toHaveBeenCalledWith(UserProfileLinkRel.SETTINGS);
    });

    it('should return result', () => {
      let result = repository.getUserSettings(userProfileResource);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: responseResource }));
    });
  });

  describe('set user settings', () => {
    const userSettingsResource: UserSettingsResource = createUserSettingsResource();
    const responseResource: UserProfileResource = createUserProfileResource();
    const userSettings: UserSettings = createUserSettings();

    beforeEach(() => {
      resourceWrapper.patch.mockReturnValue(hot('a', { a: responseResource }));
    });

    it('should call resourceFactory with resource', () => {
      repository.setUserSettings(userSettingsResource, userSettings);

      expect(resourceFactory.from).toHaveBeenCalledWith(userSettingsResource);
    });

    it('should call resourceWrapper with link', () => {
      repository.setUserSettings(userSettingsResource, userSettings);

      expect(resourceWrapper.patch).toHaveBeenCalledWith(UserSettingsLinkRel.EDIT, userSettings);
    });

    it('should return result', () => {
      let result = repository.setUserSettings(userSettingsResource, userSettings);

      expect(result).not.toBeNull();
      expect(result).toBeObservable(cold('a', { a: responseResource }));
    });
  });
});
