import { BinaryFileListResource } from '@alfa-client/binary-file-shared';
import { StateResource, createEmptyStateResource } from '@alfa-client/tech-shared';
import {
  VorgangService,
  VorgangWithEingangLinkRel,
  VorgangWithEingangResource,
} from '@alfa-client/vorgang-shared';
import { Component, Input } from '@angular/core';
import { getUrl, hasLink } from '@ngxp/rest';
import { Observable, of } from 'rxjs';

@Component({
  selector: 'alfa-vorgang-detail-dateien-container',
  templateUrl: './vorgang-detail-dateien-container.component.html',
})
export class VorgangDetailDateienContainerComponent {
  @Input() set vorgangWithEingang(vorgangWithEingang: VorgangWithEingangResource) {
    this.handleVorgangChange(vorgangWithEingang);
  }

  public attachmentListStateResource$: Observable<StateResource<BinaryFileListResource>> = of(
    createEmptyStateResource<BinaryFileListResource>(),
  );
  public representationListStateResource$: Observable<StateResource<BinaryFileListResource>> = of(
    createEmptyStateResource<BinaryFileListResource>(),
  );

  public downloadPrefix: string;
  public attachmentsDownloadUri: string;

  public readonly vorgangWithEingangLinkRel = VorgangWithEingangLinkRel;

  constructor(private vorgangService: VorgangService) {}

  handleVorgangChange(vorgangWithEingang: VorgangWithEingangResource): void {
    this.downloadPrefix = vorgangWithEingang.nummer;

    this.handleAttachments(vorgangWithEingang);
    this.getRepresentationsIfExists(vorgangWithEingang);
  }

  handleAttachments(vorgangWithEingang: VorgangWithEingangResource): void {
    this.setAttachmentsDownloadUri(vorgangWithEingang);
    this.getAttachmentsIfExists(vorgangWithEingang);
  }

  setAttachmentsDownloadUri(vorgangWithEingang: VorgangWithEingangResource): void {
    if (hasLink(vorgangWithEingang, VorgangWithEingangLinkRel.DOWNLOAD_ATTACHMENTS)) {
      this.attachmentsDownloadUri = getUrl(
        vorgangWithEingang,
        VorgangWithEingangLinkRel.DOWNLOAD_ATTACHMENTS,
      );
    }
  }

  getRepresentationsIfExists(vorgangWithEingang: VorgangWithEingangResource): void {
    if (hasLink(vorgangWithEingang, VorgangWithEingangLinkRel.REPRESENTATIONS)) {
      this.representationListStateResource$ = this.vorgangService.getRepresentations();
    }
  }

  getAttachmentsIfExists(vorgangWithEingang: VorgangWithEingangResource): void {
    if (hasLink(vorgangWithEingang, VorgangWithEingangLinkRel.ATTACHMENTS)) {
      this.attachmentListStateResource$ = this.vorgangService.getAttachments();
    }
  }
}
