import { mock } from '@alfa-client/test-utils';
import { EventEmitter } from '@angular/core';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { VorgangDetailBescheidenStepButtonComponent } from './vorgang-detail-bescheiden-step-button.component';

describe('VorgangDetailBescheidenStepButtonComponent', () => {
  let component: VorgangDetailBescheidenStepButtonComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenStepButtonComponent>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [VorgangDetailBescheidenStepButtonComponent],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenStepButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('isActive', () => {
    it('return true if step equals activeStep', () => {
      component.step = 2;
      component.activeStep = 2;

      expect(component.isActive()).toBeTruthy();
    });

    it('return false if step not equals activeStep', () => {
      component.step = 1;
      component.activeStep = 2;

      expect(component.isActive()).toBeFalsy();
    });
  });

  describe('isPrevious', () => {
    it('return true if step is less than activeStep', () => {
      component.step = 1;
      component.activeStep = 2;

      expect(component.isPrevious()).toBeTruthy();
    });

    it('return false if step equals activeStep', () => {
      component.step = 2;
      component.activeStep = 2;

      expect(component.isPrevious()).toBeFalsy();
    });

    it('return false if step is greater than activeStep', () => {
      component.step = 3;
      component.activeStep = 2;

      expect(component.isPrevious()).toBeFalsy();
    });
  });

  describe('isNext', () => {
    it('return false if step is less than activeStep', () => {
      component.step = 1;
      component.activeStep = 2;

      expect(component.isNext()).toBeFalsy();
    });

    it('return false if step equals activeStep', () => {
      component.step = 2;
      component.activeStep = 2;

      expect(component.isNext()).toBeFalsy();
    });

    it('return true if step is greater than activeStep', () => {
      component.step = 3;
      component.activeStep = 2;

      expect(component.isNext()).toBeTruthy();
    });
  });

  describe('clickHandler', () => {
    const isPreviousMock = jest.fn();
    beforeEach(() => {
      component.isPrevious = isPreviousMock;
      component.activeStepChange = <any>mock(EventEmitter);
    });

    it('should emit step', () => {
      isPreviousMock.mockReturnValue(true);

      component.clickHandler(1);

      expect(component.activeStepChange.emit).toHaveBeenCalledWith(1);
    });

    it('should not emit step', () => {
      isPreviousMock.mockReturnValue(false);

      component.clickHandler(1);

      expect(component.activeStepChange.emit).not.toHaveBeenCalled();
    });
  });
});
