import {
  BescheidLinkRel,
  BescheidResource,
  BescheidSendBy,
  BescheidService,
} from '@alfa-client/bescheid-shared';
import { StateResource, isLoaded, notHasLink } from '@alfa-client/tech-shared';
import { Component, OnInit } from '@angular/core';
import { Observable, filter, map, tap } from 'rxjs';
import { BescheidenFormService } from '../../../bescheiden.formservice';

@Component({
  selector: 'alfa-vorgang-detail-bescheiden-bescheid-versenden',
  templateUrl: './vorgang-detail-bescheiden-bescheid-versenden.component.html',
})
export class VorgangDetailBescheidenBescheidVersendenComponent implements OnInit {
  public bescheidResource$: Observable<BescheidResource>;

  public readonly formServiceClass = BescheidenFormService;
  public readonly sendBy = BescheidSendBy;

  public bescheidLinkRel = BescheidLinkRel;

  constructor(
    public formService: BescheidenFormService,
    private readonly bescheidService: BescheidService,
  ) {}

  ngOnInit(): void {
    this.bescheidResource$ = this.bescheidService.getBescheidDraft().pipe(
      filter(isLoaded),
      map((stateResource: StateResource<BescheidResource>) => stateResource.resource),
      tap((bescheidResource: BescheidResource) => this.initSendBy(bescheidResource)),
    );
  }

  initSendBy(bescheidResource: BescheidResource): void {
    if (notHasLink(bescheidResource, BescheidLinkRel.BESCHEIDEN_UND_SENDEN)) {
      this.formService.setSendBy(BescheidSendBy.MANUAL);
    }
  }
}
