package de.ozgcloud.alfa.bescheid;

import org.apache.commons.lang3.StringUtils;
import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(BescheidController.PATH)
@RequiredArgsConstructor
public class BescheidController {

	static final String PATH = "/api/bescheids"; // NOSONAR

	private final BescheidModelAssembler assembler;
	private final BescheidService bescheidService;
	private final BinaryFileController binaryFileController;

	@GetMapping
	public ResponseEntity<EntityModel<Bescheid>> getDraft(@RequestParam String vorgangId) {
		return ResponseEntity.of(bescheidService.getBescheidDraft(vorgangId).map(assembler::toModel));
	}

	// TODO Sobald Grpc Schnittstelle zum Laden eines einzelnen Bescheids existiert,
	// den Endpoint auf bescheidId beschraenken und ueber den Service ->
	// den Bescheid laden
	@GetMapping("/{bescheidId}/attachments")
	public CollectionModel<EntityModel<OzgFile>> getAttachments(@PathVariable String bescheidId, @RequestParam String vorgangId) {
		var bescheid = getBescheid(bescheidId, vorgangId);
		return binaryFileController.getFiles(bescheid.getAttachments());
	}

	Bescheid getBescheid(String bescheidId, String vorgangId) {
		var bescheid = bescheidService.findByVorgangId(vorgangId)
				.filter(oneBescheid -> StringUtils.equals(oneBescheid.getId(), bescheidId))
				.findFirst();
		return bescheid.orElseThrow(() -> new TechnicalException(String.format("No Bescheid exists for id %s.", bescheidId)));
	}
	//

	@RestController
	@RequestMapping(BescheidByVorgangController.BESCHEID_BY_VORGANG_PATH)
	@RequiredArgsConstructor
	public static class BescheidByVorgangController {

		static final String BESCHEID_BY_VORGANG_PATH = "/api/vorgangs"; // NOSONAR

		private final BescheidService service;
		private final BescheidModelAssembler modelAssembler;

		@GetMapping("/{vorgangId}/bescheids")
		public CollectionModel<EntityModel<Bescheid>> getAll(@PathVariable String vorgangId) {
			var bescheids = service.findByVorgangIdSorted(vorgangId);
			return modelAssembler.toCollectionModel(bescheids.toList());
		}
	}
}