package de.ozgcloud.alfa.bescheid;

import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.bescheid.BescheidServiceGrpc.BescheidServiceBlockingStub;
import de.ozgcloud.bescheid.GrpcBescheid;
import de.ozgcloud.bescheid.GrpcBescheidManagerConfigRequest;
import de.ozgcloud.bescheid.GrpcGetAllBescheidRequest;
import de.ozgcloud.bescheid.GrpcGetBescheidDraftRequest;
import de.ozgcloud.bescheid.GrpcGetBescheidDraftResponse;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class BescheidRemoteService {

	@GrpcClient(GrpcUtil.VORGANG_MANAGER_GRPC_CLIENT)
	private BescheidServiceBlockingStub bescheidServiceStub;
	@Autowired
	private BescheidMapper bescheidMapper;

	public Optional<Bescheid> getBescheidDraft(String vorgangId) {
		var request = buildGetBescheidDraftRequest(vorgangId);
		var response = bescheidServiceStub.getBescheidDraft(request);
		return getBescheidFromResponse(response).map(bescheid -> bescheidMapper.fromGrpc(bescheid, vorgangId));
	}

	GrpcGetBescheidDraftRequest buildGetBescheidDraftRequest(String vorgangId) {
		return GrpcGetBescheidDraftRequest.newBuilder()
				.setVorgangId(vorgangId)
				.build();
	}

	Optional<GrpcBescheid> getBescheidFromResponse(GrpcGetBescheidDraftResponse response) {
		return response.hasBescheid() ? Optional.of(response.getBescheid()) : Optional.empty();
	}

	public Stream<Bescheid> findByVorgangId(String vorgangId) {
		var request = buildGetAllBescheidRequest(vorgangId);

		var response = bescheidServiceStub.getAll(request);

		return response.getBescheidList().stream().map(grpcBescheid -> bescheidMapper.fromGrpc(grpcBescheid, vorgangId));
	}

	GrpcGetAllBescheidRequest buildGetAllBescheidRequest(String vorgangId) {
		return GrpcGetAllBescheidRequest.newBuilder().setVorgangId(vorgangId).build();
	}

	// TODO Logik in den Service verschieben
	public boolean canCreateBescheidDocument() {
		var response = bescheidServiceStub.getConfig(GrpcBescheidManagerConfigRequest.newBuilder().build());
		return response.hasFeatures() && response.getFeatures().getCanCreateBescheidDocument();
	}
}