package de.ozgcloud.alfa.collaboration;

import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.GrpcUtil;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitGetResponse;
import de.ozgcloud.zufi.grpc.organisationseinheit.GrpcOrganisationsEinheitSearchRequest;
import de.ozgcloud.zufi.grpc.organisationseinheit.OrganisationsEinheitServiceGrpc.OrganisationsEinheitServiceBlockingStub;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class OrganisationsEinheitRemoteService {

	@GrpcClient(GrpcUtil.ZUFI_MANAGER_GRPC_CLIENT)
	private OrganisationsEinheitServiceBlockingStub serviceStub;

	@Autowired
	private OrganisationsEinheitHeaderMapper organisationsEinheitHeaderMapper;
	@Autowired
	private OrganisationsEinheitMapper organisationsEinheitMapper;

	public Stream<OrganisationsEinheitHeader> search(String searchBy) {
		var response = serviceStub.search(buildSearchRequest(searchBy));
		return response.getOrganisationsEinheitenList().stream().map(organisationsEinheitHeaderMapper::fromGrpc);
	}

	private GrpcOrganisationsEinheitSearchRequest buildSearchRequest(String searchBy) {
		return GrpcOrganisationsEinheitSearchRequest.newBuilder().setSearchBy(searchBy).build();
	}

	public OrganisationsEinheit getById(String id) {
		var request = buildGetByIdRequest(id);
		var response = serviceStub.getById(request);
		return getOrganisationsEinheitFromGetByIdResponse(response);
	}

	GrpcOrganisationsEinheitGetRequest buildGetByIdRequest(String organisationsEinheitId) {
		return GrpcOrganisationsEinheitGetRequest.newBuilder().setId(organisationsEinheitId).build();
	}

	OrganisationsEinheit getOrganisationsEinheitFromGetByIdResponse(GrpcOrganisationsEinheitGetResponse response) {
		return organisationsEinheitMapper.fromGrpc(response.getOrganisationsEinheit());
	}
}
