/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.apache.commons.lang3.reflect.ConstructorUtils;
import org.springframework.hateoas.Link;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.ContextualSerializer;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.NoArgsConstructor;

@NoArgsConstructor
public class LinkedUserProfileResourceSerializer extends JsonSerializer<Object> implements ContextualSerializer {

	private LinkedUserProfileResource annotation;

	private LinkedUserProfileResourceSerializer(LinkedUserProfileResource annotation) {
		this.annotation = annotation;
	}

	@Override
	public JsonSerializer<?> createContextual(SerializerProvider prov, BeanProperty property) {
		return new LinkedUserProfileResourceSerializer(property.getAnnotation(LinkedUserProfileResource.class));
	}

	String buildLink(Object id) {
		if (UserProfileUrlProvider.isConfigured()) {
			return Link.of(UserProfileUrlProvider.getUrl(getExtractor().extractId(id))).getHref();
		} else {
			return id.toString();
		}
	}

	IdExtractor<Object> getExtractor() {
		try {
			return ConstructorUtils.invokeConstructor(annotation.extractor());
		} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException e) {
			throw new TechnicalException("Error instanciating IdExtractor", e);
		}
	}

	@Override
	public void serialize(Object value, JsonGenerator gen, SerializerProvider serializers) throws IOException {
		if (value instanceof Collection) {
			gen.writeStartArray();
			((Collection<?>) value).forEach(val -> writeObject(gen, buildLink(val)));
			gen.writeEndArray();
		} else {
			writeObject(gen, buildLink(value));
		}
	}

	void writeObject(JsonGenerator gen, Object value) {
		try {
			gen.writeObject(value);
		} catch (IOException e) {
			throw new TechnicalException("Error writing String to json", e);
		}
	}
}