package de.ozgcloud.alfa.common.binaryfile;

import java.util.Collection;
import java.util.Collections;
import java.util.Optional;

import jakarta.validation.ConstraintValidator;
import jakarta.validation.ConstraintValidatorContext;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
public class UploadBinaryFileContentTypeValidator implements ConstraintValidator<BinaryFileContentTypeConstraint, UploadBinaryFileRequest> {

	private final BinaryFileProperties properties;

	@Override
	public boolean isValid(UploadBinaryFileRequest uploadRequest, ConstraintValidatorContext context) {
		var contentTypes = getConfiguredContentTypes(uploadRequest.getField());
		return contentTypes.isEmpty() || contentTypes.contains(uploadRequest.getContentType());
	}

	Collection<String> getConfiguredContentTypes(String field) {
		return Optional.ofNullable(properties.getContentTypes().get(field)).orElse(Collections.emptyList());
	}
}
