package de.ozgcloud.alfa.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.EnumSource;
import org.junit.jupiter.params.provider.EnumSource.Mode;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.attacheditem.VorgangAttachedItem;
import de.ozgcloud.alfa.common.command.Command;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;

class DocumentHistorieProcessorTest {

	@Spy
	private DocumentHistorieProcessor processor;

	@DisplayName("Process")
	@Nested
	class TestProcess {

		private final Command command = CommandTestFactory.create();
		private final List<Command> commands = Collections.singletonList(command);

		@BeforeEach
		void mock() {
			doReturn(commands).when(processor).filterIrrelevantCommands(any());
		}

		@Test
		void shouldFilterIrrelevantCommands() {
			processor.process(commands);

			verify(processor).filterIrrelevantCommands(commands);
		}

		@Test
		void shouldReturnCommands() {
			var processedCommands = processor.process(commands);

			assertThat(processedCommands).containsExactly(command);
		}
	}

	@DisplayName("Filter irrelevant commands")
	@Nested
	class TestFilterIrrelevantCommands {

		@ParameterizedTest
		@EnumSource(mode = Mode.INCLUDE, names = { "CREATE_BESCHEID_DOCUMENT", "CREATE_BESCHEID_DOCUMENT_FROM_FILE" })
		void shouldFilterCommandWithOrder(CommandOrder order) {
			var command = CommandTestFactory.createBuilder().order(order.toString()).build();

			var commands = processor.filterIrrelevantCommands(Collections.singletonList(command));

			assertThat(commands).isEmpty();
		}

		@ParameterizedTest
		@EnumSource(mode = Mode.EXCLUDE, names = { "CREATE_BESCHEID_DOCUMENT", "CREATE_BESCHEID_DOCUMENT_FROM_FILE" })
		void shouldKeepCommandWithOrder(CommandOrder order) {
			var command = CommandTestFactory.createBuilder().order(order.toString()).build();

			var commands = processor.filterIrrelevantCommands(Collections.singletonList(command));

			assertThat(commands).containsExactly(command);
		}

		@DisplayName("should filter document related create attached item command")
		@Test
		void shouldFilterCreateAttachedItemCommand() {
			var command = CommandTestFactory.createBuilder().order(CommandOrder.CREATE_ATTACHED_ITEM.toString())
					.body(Map.of(VorgangAttachedItem.FIELD_ITEM_NAME, DocumentHistorieProcessor.DOCUMENT_ITEM_NAME))
					.build();

			var commands = processor.filterIrrelevantCommands(Collections.singletonList(command));

			assertThat(commands).isEmpty();
		}
	}
}
