package de.ozgcloud.alfa.bescheid;

import java.util.stream.Stream;

import org.springframework.stereotype.Service;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileService;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.user.UserService;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import de.xoev.xdomea.DokumentType;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
public class ExportBescheidService {
	private final BescheidService bescheidService;
	private final BinaryFileService binaryFileService;
	private final DocumentService documentService;
	private final UserService userService;

	public BescheidExportData createExportData(VorgangWithEingang vorgang) {
		var builder = BescheidExportData.builder();
		getBescheids(vorgang.getId())
				.forEach(bescheid -> addBescheidExportData(createBescheidExportInput(bescheid, vorgang), builder));
		return builder.build();
	}

	Stream<Bescheid> getBescheids(String vorgangId) {
		return bescheidService.findByVorgangIdSorted(vorgangId).filter(bescheid -> BescheidStatus.SENT == bescheid.getStatus());
	}

	BescheidExportInput createBescheidExportInput(Bescheid bescheid, VorgangWithEingang vorgang) {
		return BescheidExportInput.builder()
				.bescheid(bescheid)
				.organisationseinheitenId(vorgang.getOrganisationseinheitenID())
				.files(Stream.concat(Stream.of(getDocument(bescheid)), getAttachments(bescheid)).toList()).build();
	}

	Stream<OzgFile> getAttachments(Bescheid bescheid) {
		return binaryFileService.getFiles(bescheid.getAttachments());
	}

	void addBescheidExportData(BescheidExportInput input, BescheidExportData.BescheidExportDataBuilder builder) {
		builder.dokumentType(buildDokumentType(input))
				.files(input.files());
	}

	OzgFile getDocument(Bescheid bescheid) {
		var document = documentService.getDocument(bescheid.getBescheidDocument());
		return binaryFileService.getFile(document.getFileId());
	}

	DokumentType buildDokumentType(BescheidExportInput input) {
		var builder = DokumentTypeBuilder.builder();
		builder.withBescheid(input.bescheid())
				.withFiles(input.files())
				.withFullName(getFullName(input.bescheid()))
				.withOrganisationseinheitenId(input.organisationseinheitenId());
		return builder.build();
	}

	String getFullName(Bescheid bescheid) {
		return userService.getById(bescheid.getSentInfo().getSentBy()).getFullName();
	}

}
