package de.ozgcloud.alfa.vorgang;

import java.util.Optional;
import java.util.UUID;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import de.xoev.xdomea.AkteType;
import de.xoev.xdomea.AllgemeineMetadatenType;
import de.xoev.xdomea.IdentifikationObjektType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.VorgangType;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Service
public class ExportVorgangService {

	private final VorgangService vorgangService;

	private final VorgangTypeCreator vorgangTypeCreator;
	private final KopfCreator kopfCreator;

	public VorgangWithEingang getVorgang(String vorgangId) {
		return vorgangService.findVorgangWithEingang(vorgangId);
	}

	public VorgangType createVorgangType(VorgangWithEingang vorgangWithEingang) {
		return vorgangTypeCreator.create(vorgangWithEingang);
	}

	public NkAbgabeType createKopf(VorgangWithEingang vorgangWithEingang) {
		return kopfCreator.createKopf(vorgangWithEingang);
	}

	public AkteType createAkteType(VorgangWithEingang vorgangWithEingang) {
		var akteType = new AkteType();
		akteType.setIdentifikation(createIdentifikationObjektType());
		akteType.setAllgemeineMetadaten(createAllgemeineMetadatenType(vorgangWithEingang));
		return akteType;
	}

	IdentifikationObjektType createIdentifikationObjektType() {
		var identifikationObjektType = new IdentifikationObjektType();
		identifikationObjektType.setID(UUID.randomUUID().toString());
		return identifikationObjektType;
	}

	AllgemeineMetadatenType createAllgemeineMetadatenType(VorgangWithEingang vorgangWithEingang) {
		var allgemeineMetadatenType = new AllgemeineMetadatenType();
		allgemeineMetadatenType.setKennzeichen(Optional.ofNullable(vorgangWithEingang.getAktenzeichen()).orElse(StringUtils.EMPTY));
		return allgemeineMetadatenType;
	}
}
