package de.ozgcloud.alfa.vorgang;

import java.time.ZoneOffset;
import java.time.ZonedDateTime;

import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.DateConverter;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.export.XdomeaProperties;
import de.xoev.xdomea.BehoerdenkennungType;
import de.xoev.xdomea.Code;
import de.xoev.xdomea.KontaktType;
import de.xoev.xdomea.NachrichtentypCodeType;
import de.xoev.xdomea.NkAbgabeType;
import de.xoev.xdomea.OrganisationseinheitType;
import de.xoev.xdomea.SystemType;
import io.micrometer.common.util.StringUtils;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class KopfCreator {

	static final String NACHRICHTENTYP_CODE_TYPE_LIST_URI = "urn:xoev-de:xdomea:codeliste:nachrichtentyp";
	static final String NACHRICHTENTYP_CODE_TYPE_LIST_VERSION_ID = "2.0";
	static final String NACHRICHTENTYP_ABGABE_ABGABE_TYPE_CODE = "0401";
	static final String PRODUKT_NAME = "OZG-Cloud";

	private final XdomeaProperties xdomeaProperties;

	public NkAbgabeType createKopf(VorgangWithEingang vorgang) {
		var nkAbgabeType = new NkAbgabeType();
		nkAbgabeType.setProzessID(UUIDConverter.fromObjectId(vorgang.getId()));
		nkAbgabeType.setNachrichtentyp(createNachrichtentyp());
		nkAbgabeType.setErstellungszeitpunkt(DateConverter.toXmlGregorianCalendar(ZonedDateTime.now(ZoneOffset.UTC)));
		nkAbgabeType.setAbsender(createAbsender(vorgang.getEingang().getZustaendigeStelle().getOrganisationseinheitenId()));
		nkAbgabeType.setEmpfaenger(createKontaktType());
		nkAbgabeType.setSendendesSystem(createSendendesSystem());
		nkAbgabeType.setImportbestaetigung(true);
		nkAbgabeType.setEmpfangsbestaetigung(true);
		return nkAbgabeType;
	}

	NachrichtentypCodeType createNachrichtentyp() {
		var nachrichtentypCode = new NachrichtentypCodeType();
		nachrichtentypCode.setCode(NACHRICHTENTYP_ABGABE_ABGABE_TYPE_CODE);
		nachrichtentypCode.setListURI(NACHRICHTENTYP_CODE_TYPE_LIST_URI);
		nachrichtentypCode.setListVersionID(NACHRICHTENTYP_CODE_TYPE_LIST_VERSION_ID);
		return nachrichtentypCode;
	}

	KontaktType createAbsender(String organisationseinheitId) {
		var absender = createKontaktType();
		absender.setOrganisationseinheit(createOrganisationseinheitType(organisationseinheitId));
		return absender;
	}

	KontaktType createKontaktType() {
		var kontakt = new KontaktType();
		if (StringUtils.isNotBlank(xdomeaProperties.getBehoerdenschluessel())) {
			kontakt.setBehoerdenkennung(createBehoerdenkennung());
		}
		return kontakt;
	}

	OrganisationseinheitType createOrganisationseinheitType(String organisationseinheitId) {
		var organisationseinheit = new OrganisationseinheitType();
		organisationseinheit.setName(organisationseinheitId);
		return organisationseinheit;
	}

	BehoerdenkennungType createBehoerdenkennung() {
		var behoerdenkennungType = new BehoerdenkennungType();
		behoerdenkennungType.setBehoerdenschluessel(createBehoerdenschluessel());
		return behoerdenkennungType;
	}

	Code createBehoerdenschluessel() {
		var behoerdenschluessel = new Code();
		behoerdenschluessel.setCode(xdomeaProperties.getBehoerdenschluessel());
		behoerdenschluessel.setListURI(xdomeaProperties.getBehoerdenschluesselUri());
		behoerdenschluessel.setListVersionID(xdomeaProperties.getBehoerdenschluesselVersion());
		return behoerdenschluessel;
	}

	SystemType createSendendesSystem() {
		var sendendesSystem = new SystemType();
		sendendesSystem.setProduktname(PRODUKT_NAME);
		return sendendesSystem;
	}

}
