package de.ozgcloud.alfa.export;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.MockedStatic;

import de.ozgcloud.alfa.common.ExportFilenameGenerator;
import de.ozgcloud.alfa.common.AlfaTestUtils;
import de.ozgcloud.alfa.common.UUIDConverter;
import de.ozgcloud.alfa.common.file.OzgFile;
import de.ozgcloud.alfa.common.file.OzgFileTestFactory;

class ExportFilenameGeneratorTest {

	private MockedStatic<UUIDConverter> uuidConverter;

	@Nested
	class TestGenerateExportFilename {

		private static final String UUID = "64a820d3-6285-172a-c028-0000000026d0";

		private final OzgFile ozgFile = OzgFileTestFactory.create();

		@BeforeEach
		void init() {
			uuidConverter = mockStatic(UUIDConverter.class);
			uuidConverter.when(() -> UUIDConverter.fromObjectId(ozgFile.getId().toString())).thenReturn(UUID);
		}

		@AfterEach
		void cleanup() {
			uuidConverter.close();
		}

		@Test
		void shouldConvertObjectId() {
			callGenerator();

			uuidConverter.verify(() -> UUIDConverter.fromObjectId(ozgFile.getId().toString()));
		}

		@Test
		void shouldGenerate() {
			var filename = callGenerator();

			assertThat(filename).matches(String.format("^%s_%s$", AlfaTestUtils.UUID_REGEX, ozgFile.getName()));
		}

		private String callGenerator() {
			return ExportFilenameGenerator.generateExportFilename(ozgFile);
		}
	}

}